import numpy as np
from skimage.filters import threshold_multiotsu
from skimage.measure import label, regionprops
from skimage.morphology import binary_dilation, binary_erosion, disk
from scipy.ndimage.morphology import binary_fill_holes


def get_edge_mask(true_fg, dil_radius, ero_radius):
    selem_dilation = disk(dil_radius)
    selem_erosion = disk(ero_radius)
    edge_dilation = binary_dilation(true_fg, selem_dilation)
    edge_erosion = binary_erosion(true_fg, selem_erosion)

    return np.logical_not(np.logical_or(
        np.logical_not(edge_dilation),
        edge_erosion,
    ))


def extract_mask(img, mask):
    masked_img = np.zeros_like(img)
    masked_img[mask] = img[mask]
    return masked_img


def new_edge_threshold(img, mask, true_fg):
    thresholds = threshold_multiotsu(img[mask], classes=2)
    return np.logical_or(np.digitize(img, thresholds), true_fg)


def extract_foreground_biofilms(img, area_threshold=9000):
    thresholds = threshold_multiotsu(img, classes=3)
    true_fg = np.digitize(img, [thresholds[0]])

    mask_edge = get_edge_mask(true_fg, 100, 20)
    masked_img = extract_mask(img, mask_edge)
    new_true_fg = new_edge_threshold(masked_img, mask_edge, true_fg)

    new_mask_edge = get_edge_mask(new_true_fg, 50, 10)
    new_masked_img = extract_mask(img, new_mask_edge)
    last_true_fg = new_edge_threshold(
        new_masked_img, new_mask_edge, new_true_fg,
    )

    labels = label(last_true_fg)
    props = regionprops(labels, img)

    areas = np.asarray([prop.area for prop in props])
    mask = np.zeros_like(labels)
    inds = np.arange(len(areas))[areas > area_threshold]
    for i, ind in enumerate(inds):
        prop = props[ind]
        mask[binary_fill_holes(labels == prop.label)] = i + 1

    return mask


def extract_foreground(img):
    """
    Extracts the single largest object from grayscale image img.
    Returns a boolean mask and a skimage RegionProperty for that object.
    """
    thresholds = threshold_multiotsu(img, classes=3)
    true_fg = np.digitize(img, [thresholds[0]])

    mask_edge = get_edge_mask(true_fg, 100, 20)
    masked_img = extract_mask(img, mask_edge)
    new_true_fg = new_edge_threshold(masked_img, mask_edge, true_fg)

    new_mask_edge = get_edge_mask(new_true_fg, 50, 10)
    new_masked_img = extract_mask(img, new_mask_edge)
    last_true_fg = new_edge_threshold(
        new_masked_img, new_mask_edge, new_true_fg,
    )

    labels = label(last_true_fg)
    props = regionprops(labels, img)

    areas = np.asarray([prop.area for prop in props])
    ind = areas.argmax()

    prop = props[ind]
    mask = binary_fill_holes(labels == prop.label)

    return mask, prop


def extract_biofilm(img):
    mask, _ = extract_foreground(img)
    return fill_border_annulus(mask)


def fill_border_annulus(mask):
    # we reflect the mask along all borders fill holes and use only the
    # original part of that image. Then the annulus must be filled.
    x, y = mask.shape[0], mask.shape[1]
    reflection = np.zeros((x * 3, y * 3), dtype=mask.dtype)
    reflection[:x, y:2*y] = mask[::-1, :]
    reflection[2*x:, y:2*y] = mask[::-1, :]

    reflection[x:2*x, :y] = mask[:, ::-1]
    reflection[x:2*x, 2*y:] = mask[:, ::-1]

    reflection[x:2*x, y:2*y] = mask
    reflection = binary_fill_holes(reflection)
    mask = reflection[x:2*x, y:2*y]
    return mask



# -*- coding: utf-8 -*-


def some_function(first: int, second: int) -> int:
    """
    We use this function as an example for some real logic.

    This is how you can write a doctest:

    .. code:: python

        >>> some_function(2, 3)
        5

    Enjoy!
    """
    return first + second



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gatoniel_utils', 'gatoniel_utils.segmentation']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.20.1,<2.0.0', 'scikit-image>=0.18.1,<0.19.0', 'scipy>=1.6.1,<2.0.0']

setup_kwargs = {
    'name': 'gatoniel-utils',
    'version': '0.1.0',
    'description': 'Library with utils and helpers for myself.',
    'long_description': '# gatoniel-utils\n\n[![Build Status](https://github.com/gatoniel/gatoniel-utils/workflows/test/badge.svg?branch=master&event=push)](https://github.com/gatoniel/gatoniel-utils/actions?query=workflow%3Atest)\n[![codecov](https://codecov.io/gh/gatoniel/gatoniel-utils/branch/master/graph/badge.svg)](https://codecov.io/gh/gatoniel/gatoniel-utils)\n[![Python Version](https://img.shields.io/pypi/pyversions/gatoniel-utils.svg)](https://pypi.org/project/gatoniel-utils/)\n[![wemake-python-styleguide](https://img.shields.io/badge/style-wemake-000000.svg)](https://github.com/wemake-services/wemake-python-styleguide)\n\nLibrary with utils and helpers for myself.\n\n\n## Features\n\n- Fully typed with annotations and checked with mypy, [PEP561 compatible](https://www.python.org/dev/peps/pep-0561/)\n- Add yours!\n\n\n## Installation\n\n```bash\npip install gatoniel-utils\n```\n\n\n## Example\n\nShowcase how your project can be used:\n\n```python\nfrom gatoniel_utils.example import some_function\n\nprint(some_function(3, 4))\n# => 7\n```\n\n## License\n\n[MIT](https://github.com/gatoniel/gatoniel-utils/blob/master/LICENSE)\n\n\n## Credits\n\nThis project was generated with [`wemake-python-package`](https://github.com/wemake-services/wemake-python-package). Current template version is: [0c5619079cb74e7dc72e1f18e9176b3b3a1a843f](https://github.com/wemake-services/wemake-python-package/tree/0c5619079cb74e7dc72e1f18e9176b3b3a1a843f). See what is [updated](https://github.com/wemake-services/wemake-python-package/compare/0c5619079cb74e7dc72e1f18e9176b3b3a1a843f...master) since then.\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gatoniel/gatoniel-utils',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
